//Created by Takamitsu Mizutori on 9/Aug/2018

#include <Servo.h>
#include "cli.h"
#define printLog(x,y) (Serial.print(x), Serial.println(y))

const int TEST_LED = 7;

//DCモーターコントロール
const int SPEED_PIN = 4;
const int IN1 = 5;
const int IN2 = 6;


//センサーから読み取った値を格納するための変数。0〜1023までの整数が入ってくるから int型で宣言
int val1 = 0;

const int LED1 = 9;
const int LED2 = 10;
const int LED3 = 11;
const int LED4 = 12;
const int LED5 = 13;

int counter = 0;


//サーボ系
const int SERVO_CTRL = 2; //D2ピンをサーボの信号線として設定
const int deg0msec = 600; // msec.
const int deg180msec = 2350; // msec.
int microSec = deg0msec;
Servo servo;

/*
  コマンドに対応して実行される関数の定義
 */
int move(char **args);
int moveBack(char **args);
int stop(char **args);
int ping(char **args);
int lightOn(char **args);
int lightOff(char **args);
int turn(char **args);

/*
  コマンドのリストと、それに続いて対応する関数ポインターのリスト。（この２つのリストでジャンプテーブルのようなものを作る）
 */
const char *commandList[] = {
  "move",
  "turn"
};

int (*functionPointerList[]) (char **) = {
  &move,
  &turn
};

/*
  commandListを定義したのと同じスコープでなければsizeofが正しく機能しないのでクラスレベルで関数にしておく(executeCommand関数の中で動的にサイズ取得はしない）
*/
int numberOfCommands() {  
  return sizeof(commandList) / sizeof(char *);
}


/*
  ここにコマンドの実装を置く
*/

//前進
int move(char **args){
  int direction = atoi(args[1]);
  int time = 5;
  int speed = 255;
  
  if (args[2] != NULL) {
    time = atoi(args[2]);
  }

  if (args[3] != NULL) {
    speed = atoi(args[3]);
  }

  servo.write(90);
  delay(500);

  moveFunction(direction, speed);  
  delay(time * 1000);
  moveFunction(0, 0);
  
  return 1;
}

void moveFunction(int direction, int speed){
  pinMode(IN1,OUTPUT); //信号用ピン IN1(5番ピン)
  pinMode(IN2,OUTPUT); //信号用ピン IN2(6番ピン)

  //IN2 LOW -> Forward
  //IN1 LOW -> Back
  if(direction == 0){ //Stop
    analogWrite(IN1, 0);
    analogWrite(IN2, 0);
  }else if(direction == 1){ //Forward
    analogWrite(IN1, 0);
    analogWrite(IN2, 255);
  }else if(direction == 2){ //Backward
    analogWrite(IN1, 255);
    analogWrite(IN2, 0);
  }  

  //valが大きいほど出力値も大きくなる
  analogWrite(SPEED_PIN , speed); //出力値:1~255

  pinMode(TEST_LED, OUTPUT);
  digitalWrite(TEST_LED, HIGH);
}

int turn(char **args){
  int direction = atoi(args[1]); //0: Right, 1: Left
  seeFunction(direction);    

  pinMode(TEST_LED, OUTPUT);
  digitalWrite(TEST_LED, HIGH);
  
  return 1;
}

//サーボ系の関数
void headFunction(int degree) {
  servo.write(degree); // サーボの角度を90°に設定
  delay(1000);
}

void seeFunction(int see_direction) {
  // 2 -> front(90deg)
  // 1 -> left(30deg)
  // 0 -> right(150deg)
  
  if (see_direction == 1) { // left(30deg)
    servo.write(150);
  } else if (see_direction == 0) { // right(150deg)
    servo.write(30);
  } else { //front(90deg);
    servo.write(90);
  } 
}

void aroundFunction(){
  for (int num = 10; num < 170; num++) {
    servo.write(num);
    delay( 1 );
  }
  for (int num = 170; num > 10; num--) {
    servo.write(num);
    delay( 1 );
  }
  for (int num = 10; num < 90; num++) {
    servo.write(num);
    delay( 1 );
  }
}


/*
  初期化関数
*/
void setup()
{
  Serial.begin(115200);
  pinMode(LED1,OUTPUT);
  pinMode(LED2,OUTPUT);
  pinMode(LED3,OUTPUT);
  pinMode(LED4,OUTPUT);
  pinMode(LED5,OUTPUT);

  digitalWrite(LED1,LOW);
  digitalWrite(LED2,LOW);
  digitalWrite(LED3,LOW);
  digitalWrite(LED4,LOW);
  digitalWrite(LED5,LOW);

  //Servoのポート設定
  pinMode(SERVO_CTRL,OUTPUT);
  servo.attach(SERVO_CTRL);

}

/*
  Mainループ
*/
void loop()
{
 
  char *commandLine;
  char **args;
  int status;
  commandLine = getCommandLineFromSerialPort();
  if (commandLine != NULL) {
    args = parseCommand(commandLine);
    if(args != NULL){
      status = executeCommand(args, commandList, functionPointerList, numberOfCommands());
      free(args); //argsがparseCommandの中でallocされているのでここでフリーする (commandLineは毎回アロケートされていないのでfreeする必要はない)
    }    
  }

  
  mic_LED();
}


void mic_LED(){

      //arduinoのA0番ピンの値を読んで、val1に格納するよという意味
   val1 = analogRead(A0);
   //マイクセンサーは、2.5vを中心に波の形の変化をする。
   //で、analogRead()は0〜1023（0v〜5vの電圧変化）なので、真ん中の512を引いて、
   //その値の差=音量を取得する。普通に引き算するだけだと、マイナスの負の値も
   //出てくるので、abs()で絶対値に変えて、val1に再び格納しなおす。
   val1 = abs(512-val1);
  
   
  counter++;  
  if(counter > 1000){
    counter = 0;
  }else{
    return;
  }

/*
  digitalWrite(LED1,LOW); 
     digitalWrite(LED2,LOW);
     digitalWrite(LED3,LOW);
     digitalWrite(LED4,LOW);
     digitalWrite(LED5,LOW);
     */


   

   printLog("val1", val1);

   
   //
   if(0 < val1 && val1 <= 20){ 
    printLog("level", 1);
     digitalWrite(LED1,HIGH); 
     digitalWrite(LED2,LOW);
     digitalWrite(LED3,LOW);
     digitalWrite(LED4,LOW);
     digitalWrite(LED5,LOW);
     delay(1);
   }else if(20 < val1 && val1 <=  30){ 
    printLog("level", 2);
     digitalWrite(LED1,HIGH); 
     digitalWrite(LED2,HIGH);
     digitalWrite(LED3,LOW);
     digitalWrite(LED4,LOW);
     digitalWrite(LED5,LOW);
     delay(1);
   }else if(30 < val1 && val1 <= 50){ 
    printLog("level", 3);
     digitalWrite(LED1,HIGH); 
     digitalWrite(LED2,HIGH);
     digitalWrite(LED3,HIGH);
     digitalWrite(LED4,LOW);
     digitalWrite(LED5,LOW);
     delay(1);
   }else if(50 < val1 && val1 <= 70){ 
    printLog("level", 4);
     digitalWrite(LED1,HIGH); 
     digitalWrite(LED2,HIGH);
     digitalWrite(LED3,HIGH);
     digitalWrite(LED4,HIGH);
     digitalWrite(LED5,LOW);
     delay(1);
   }else if(70 < val1){
    printLog("level", 5);
     digitalWrite(LED1,HIGH); 
     digitalWrite(LED2,HIGH);
     digitalWrite(LED3,HIGH);
     digitalWrite(LED4,HIGH);
     digitalWrite(LED5,HIGH);
     delay(1);
   }else{
    printLog("level", 0);
     digitalWrite(LED1,LOW);
     digitalWrite(LED2,LOW);
     digitalWrite(LED3,LOW);
     digitalWrite(LED4,LOW);
     digitalWrite(LED5,LOW);
   }
}



